@@ -1,14 +1,14 @@
 package XML::Struct::Reader;
-# ABSTRACT: Read XML streams into XML data structures
-our $VERSION = '0.23'; # VERSION
-
 use strict;
+
 use Moo;
 use Carp qw(croak);
 our @CARP_NOT = qw(XML::Struct);
 use Scalar::Util qw(blessed);
 use XML::Struct;
 
+our $VERSION = '0.23';
+
 has whitespace => (is => 'rw', default => sub { 0 });
 has attributes => (is => 'rw', default => sub { 1 });
 has path       => (is => 'rw', default => sub { '*' }, isa => \&_checkPath);
@@ -101,10 +101,11 @@ sub _checkPath {
 }
 
 sub _nameMatch {
-   return ($_[0] eq '*' or $_[0] eq $_[1]); 
+    return ($_[0] eq '*' or $_[0] eq $_[1]); 
 }
 
-sub readNext { # TODO: use XML::LibXML::Reader->nextPatternMatch for more performance
+# TODO: use XML::LibXML::Reader->nextPatternMatch for more performance
+sub readNext { 
     my $self   = shift;
     my $stream = blessed $_[0] ? shift() : $self->stream;
     my $path   = defined $_[0] ? _checkPath($_[0]) : $self->path;
@@ -235,21 +236,14 @@ sub readContent {
 }
 
 1;
-
 __END__
 
-=pod
-
 =encoding UTF-8
 
 =head1 NAME
 
 XML::Struct::Reader - Read XML streams into XML data structures
 
-=head1 VERSION
-
-version 0.23
-
 =head1 SYNOPSIS
 
     my $reader = XML::Struct::Reader->new( from => "file.xml" );
@@ -400,15 +394,4 @@ L<XML::Struct::Simple> for documentation.
 
 =back
 
-=head1 AUTHOR
-
-Jakob Voß
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2014 by Jakob Voß.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
 =cut
