@@ -1,11 +1,10 @@
 package XML::Struct::Simple;
-# ABSTRACT: Transform MicroXML data structures into simple (unordered) form
-our $VERSION = '0.23'; # VERSION
-
 use strict;
 use Moo;
 use List::Util qw(first);
 
+our $VERSION = '0.24';
+
 has root       => (is => 'rw', default => sub { 0 });
 has attributes => (is => 'rw', default => sub { 1 });
 has content    => (is => 'rw', default => sub { 'content' });
@@ -99,21 +98,14 @@ sub _push_hash {
 
 
 1;
-
 __END__
 
-=pod
-
 =encoding UTF-8
 
 =head1 NAME
 
 XML::Struct::Simple - Transform MicroXML data structures into simple (unordered) form
 
-=head1 VERSION
-
-version 0.23
-
 =head1 SYNOPSIS
 
     my $converter = XML::Struct::Simple->new( root => 'record' );
@@ -165,15 +157,17 @@ C<remove> will first remove attributes, so the following three are equivalent:
 Option C<KeyAttr>, C<ForceArray>, and other fetures of L<XML::Simple> not
 supported (yet).
 
-=head1 AUTHOR
+=head1 METHODS
 
-Jakob Voß
+=head2 transform( $element )
 
-=head1 COPYRIGHT AND LICENSE
+Transform XML given as array reference (MicroXML) to XML as hash reference
+(SimpleXML) as configured.
 
-This software is copyright (c) 2014 by Jakob Voß.
+=head2 FUNCTIONS
 
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
+=head2 removeXMLAttr( $element )
+
+Recursively remove XML attributes from XML given as array reference (MicroXML).
 
 =cut
